import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D #Braucht man für 3D Plots


class Vertex:
    def __init__(self, coords = np.zeros(3), edge = None):
        self.edge = edge
        self.coords = coords
        
    def __str__(self):
        return str(self.coords)

class Face:
    def __init__(self, halfedge = None):
        self.edge = halfedge

class Halfedge:
    def __init__(self, nextEdge = None, twinEdge = None, point = None, face = None):
        self.nextEdge = nextEdge #Naechste Kante
        self.twinEdge = twinEdge #Zwillingskante
        self.vertex = point #Der Punkt auf den die Kante zeigt
        self.face = face #Zugehoerige Flaeche
    
class Mesh:
    def __init__(self):

        self.vertices = []
        self.faces = []
        self.edges = []
            
    def coords(self):
        #Zuerst Liste anlegen:
        coords = []
        for v in self.vertices:
            coords.append(v.coords)
        coords = np.array(coords)
        return coords
        
    def plot(self, color = 'b', points = True, pointnum = True):
        figure = plt.figure()
        ax = figure.add_subplot(111, projection='3d')
        ax.set_aspect(1)
        
        if points:
            ax.scatter3D(self.coords()[:,0], self.coords()[:,1], self.coords()[:,2], color = 'r')
            
        if pointnum:
            n = 0
            for v in self.vertices:
                ax.text(self.coords()[n,0]+0.05, self.coords()[n,1],self.coords()[n,2],n, fontsize=9)
                n = n+1
    
        for face in self.faces:
            edge = face.edge
            #Die Punkte fuer die erste Kante waehlen:
            v0 = edge.vertex 
            v1 = edge.nextEdge.vertex
            #Die erste Kante plotten:
            ax.plot3D([v0.coords[0],v1.coords[0]],[v0.coords[1], v1.coords[1]],[v0.coords[2], v1.coords[2]], color = color)
            #Eine Kante weiter gehen:
            edge = edge.nextEdge
            v0 = v1
            v1 = edge.nextEdge.vertex
            while (not v0 is face.edge.vertex): #Ueberpruefen, ob wir bei der Anfangskante angelangt sind.
                #Kante plotten:
                ax.plot([v0.coords[0],v1.coords[0]],[v0.coords[1], v1.coords[1]],[v0.coords[2], v1.coords[2]], color = color)
                #Eine Kante weiter gehen:
                edge = edge.nextEdge
                v0 = v1
                v1 = edge.vertex
        
        ax.set_xlabel('X axis')
        ax.set_ylabel('Y axis')
        ax.set_zlabel('Z axis')
        
        plt.show()
        return figure
 
class Quad(Mesh):
    
    def __init__(self, points = [[0,0,0],[1,0,0],[1,1,0],[0,1,0]]):
        Mesh.__init__(self) #Ruft den Mesh Initialisator auf.
        
        self.faces.append(Face())
        
        #Punkte anlegen
        for i in range(4):
            self.vertices.append(Vertex(points[i]))
        
        #Innen-Kanten anlegen:
        self.edges.append(Halfedge(None,None,self.vertices[0],self.faces[0]))
        self.edges.append(Halfedge(self.edges[0],None,self.vertices[3],self.faces[0]))
        self.edges.append(Halfedge(self.edges[1],None,self.vertices[2],self.faces[0]))
        self.edges.append(Halfedge(self.edges[2],None,self.vertices[1],self.faces[0]))
        self.edges[0].nextEdge = self.edges[3]
        
        #Jedem Punkt die ausgehende Innen-Kante zuordnen:
        self.vertices[0].edge = self.edges[3]
        self.vertices[1].edge = self.edges[2]
        self.vertices[2].edge = self.edges[1]            
        self.vertices[3].edge = self.edges[0]
        
        #Außenkanten setzen:
        for i in range(4):
            self.edges.append(Halfedge(None,self.edges[i],self.vertices[3-i],None))
                
        #Zwillingskanten zuordnen:
        for i in range(4):
            self.edges[i].twinedge = self.edges[i + 4]
        
        #Dem face eine Halfedge zuordnen:
        self.faces[0].edge = self.edges[0]
